"""Example settings file"""

REVEAL_META = {
    # Title of the slide
    "title": "revelation",
    # Author in the metadata of the slide
    "author": "Humberto Rocha",
    # Description in the metadata of the slide
    "description": "A revelation example presentation",
}

# Reveal markdown slide separator
REVEAL_SLIDE_SEPARATOR = "---"
REVEAL_VERTICAL_SLIDE_SEPARATOR = "---~"

# Themes
# beige, black, blood, league, moon, night, serif, simple, sky,
# solarized, white, dracula
REVEAL_THEME = "sky"

REVEAL_CONFIG = {
    "controls": False,
    "slideNumber": True,
    "hash": True,
}



import tarfile
import zipfile
from dataclasses import dataclass
from pathlib import Path

import pytest

from revelation import Revelation


@dataclass
class Presentation:
    parent: Path

    @property
    def root(self):
        return self.parent / "presentation"

    @property
    def file(self):
        return self.root / "slides.md"

    @property
    def config(self):
        return self.root / "config.py"

    @property
    def media(self):
        return self.root / "media"


@pytest.fixture
def presentation(tmp_path: Path) -> Presentation:
    presentation = Presentation(tmp_path)

    presentation.root.mkdir()
    presentation.media.mkdir()
    presentation.file.write_text("# Test", "utf8")

    reveal_meta = {
        "title": "Test Title",
        "author": "Test Author",
        "description": "Test description",
    }

    presentation.config.write_text(f"REVEAL_META = {reveal_meta}", "utf8")

    return presentation


@pytest.fixture
def presentation_zip(presentation: Presentation) -> Path:
    presentation_zip = presentation.parent / "presentation.zip"

    with zipfile.ZipFile(presentation_zip, "w") as fp:
        arcname = Path("presentation_zip")
        fp.write(presentation.root, arcname)

        for file in presentation.root.iterdir():
            fp.write(file, arcname / file.name)

    return presentation_zip


@pytest.fixture
def presentation_tar(presentation: Presentation) -> Path:
    presentation_tar = presentation.parent / "presentation.tar.gz"

    with tarfile.open(presentation_tar, "w:gz") as fp:
        arcname = Path("presentation_tar")
        fp.add(presentation.root, arcname)

    return presentation_tar


@pytest.fixture
def revelation(presentation: Presentation) -> Revelation:
    return Revelation(presentation.file, media=presentation.media)



"""Cli tool to handle revelation commands"""

import glob
import shutil
from pathlib import Path
from typing import Any

import typer
from typer.main import get_command
from werkzeug.serving import run_simple

import revelation
from revelation import Revelation
from revelation.cli_types import (
    ConfigFile,
    DebugFlag,
    MediaDir,
    OutputFile,
    OutputFolder,
    OverwriteOutputFlag,
    RevealUrl,
    ServerPort,
    StyleOverrideFile,
    ThemeDir,
)
from revelation.constants import REVEAL_URL, REVEALJS_DIR
from revelation.utils import (
    download_file,
    extract_file,
    make_presentation,
    move_and_replace,
)

cli = typer.Typer()
echo = typer.echo


def error(message: str) -> None:
    typer.secho(f"Error: {message}", err=True, fg="red", bold=True)


def revelation_factory(
    presentation: Path,
    config: ConfigFile = None,
    media: MediaDir = None,
    theme: ThemeDir = None,
    style: StyleOverrideFile = None,
) -> Revelation:
    if presentation.is_file():
        path = presentation.parent
    else:
        error("Presentation file not found.")

        raise typer.Abort()

    if style and (not style.is_file() or not style.suffix == ".css"):
        error("Style is not a css file or does not exists.")

        raise typer.Abort()

    if not media:
        media = path / "media"

    if not media.is_dir():
        media = None

        echo("Media folder not detected, running without media.")

    if not theme:
        theme = path / "theme"

    if not theme.is_dir():
        theme = None

        echo("Theme not detected, running without custom theme.")

    if not config:
        config = path / "config.py"

    if config and not config.is_file():
        config = None

        echo("Configuration file not detected, running with defaults.")

    return Revelation(presentation, config, media, theme, style)


@cli.command()
def version() -> None:
    """
    Show version
    """
    echo(revelation.__version__)


@cli.command()
def installreveal(url: RevealUrl = REVEAL_URL) -> None:
    """
    Install or upgrade reveal.js dependency

    Receives the download url to install from a specific version or
    downloads the latest version if noting is passed
    """
    echo("Downloading reveal.js...")

    download = download_file(url)

    echo("Installing reveal.js...")

    move_and_replace(extract_file(download[0]), REVEALJS_DIR)

    echo("Installation completed!")


@cli.command()
def mkpresentation(presentation: Path) -> None:
    """Create a new revelation presentation"""
    if presentation.exists():
        error(f"'{presentation}' already exists.")

        raise typer.Abort()

    echo("Starting a new presentation...")

    make_presentation(presentation)


@cli.command()
def mkstatic(
    ctx: typer.Context,
    presentation: Path,
    *,
    config: ConfigFile = None,
    media: MediaDir = None,
    theme: ThemeDir = None,
    output_folder: OutputFolder = Path("output"),
    output_file: OutputFile = Path("index.html"),
    force: OverwriteOutputFlag = False,
    style: StyleOverrideFile = None,
) -> None:
    """Make static presentation"""

    if not REVEALJS_DIR.exists():
        echo("Reveal.js not found, running installation...")

        # Change after fix on https://github.com/tiangolo/typer/issues/102
        ctx.invoke(
            get_command(cli).get_command(ctx, "installreveal")  # type: ignore
        )

    if output_folder.is_dir():
        if force:
            shutil.rmtree(output_folder)
        else:
            error(f"'{output_folder}' already exists, use --force to override it.")

            raise typer.Abort()

    if output_folder.is_file():
        error(f"'{output_folder}' already exists and is a file.")

        raise typer.Abort()

    app = revelation_factory(presentation, config, media, theme, style)

    echo("Generating static presentation...")

    staticfolder = output_folder / "static"

    output_folder.mkdir(parents=True, exist_ok=True)

    if app.style:
        shutil.copy(app.style, output_folder / app.style.name)

    shutil.copytree(REVEALJS_DIR, staticfolder / "revealjs")

    if app.media:
        shutil.copytree(app.media, output_folder / "media")

    if app.theme:
        shutil.copytree(app.theme, output_folder / "theme")

    output_folder = output_folder.resolve()

    if not output_folder.is_dir():
        output_folder.mkdir(parents=True, exist_ok=True)

    output_file = output_folder / output_file

    with output_file.open("wb") as fp:
        fp.write(app.dispatch_request().get_data(as_text=True).encode("utf-8"))

    echo(f"Static presentation generated in {output_folder}")


@cli.command()
def start(
    ctx: typer.Context,
    presentation: Path,
    *,
    port: ServerPort = 4000,
    config: ConfigFile = None,
    media: MediaDir = None,
    theme: ThemeDir = None,
    style: StyleOverrideFile = None,
    debug: DebugFlag = False,
) -> None:
    """Start the revelation server"""

    if not REVEALJS_DIR.exists():
        echo("Reveal.js not found, running installation...")

        # Change after fix on https://github.com/tiangolo/typer/issues/102
        ctx.invoke(
            get_command(cli).get_command(ctx, "installreveal")  # type: ignore
        )

    app = revelation_factory(presentation, config, media, theme, style)

    presentation_root = presentation.parent

    echo("Starting revelation server...")

    server_args: dict[str, Any] = {
        "hostname": "localhost",
        "port": port,
        "application": app,
        "use_reloader": False,
    }

    if debug:
        server_args["use_debugger"] = True
        server_args["use_reloader"] = True
        server_args["reloader_type"] = "watchdog"
        server_args["extra_files"] = glob.glob(
            str(presentation_root / "*.md")
        ) + glob.glob(str(presentation_root / "*.css"))

    run_simple(**server_args)



from revelation.config import Config

from .conftest import Presentation


def test_config_extends_dict(presentation: Presentation) -> None:
    config = Config(presentation.config)

    assert isinstance(config, dict)


def test_config_initialize_variables(presentation: Presentation) -> None:
    config = Config(presentation.config)

    assert "REVEAL_META" in config
    assert "REVEAL_SLIDE_SEPARATOR" in config
    assert "REVEAL_THEME" in config
    assert "REVEAL_CONFIG" in config


def test_config_custom_values(presentation: Presentation) -> None:
    config = Config(presentation.config)
    meta = {
        "title": "Test Title",
        "author": "Test Author",
        "description": "Test description",
    }

    assert config["REVEAL_META"] == meta



from pathlib import Path

import pytest

from revelation.utils import extract_file, make_presentation, move_and_replace

from .conftest import Presentation


def test_helper_move_and_replace(presentation: Presentation) -> None:
    dst_dir = presentation.parent / "destination"
    dst_dir.mkdir()

    dst_file = dst_dir / "slides.md"
    dst_file.write_text("replace", "utf8")

    src_files = sorted(f.name for f in presentation.root.iterdir())

    move_and_replace(presentation.root, dst_dir)

    dst_files = sorted(f.name for f in dst_dir.iterdir())

    # The moved directory should not exist because it was moved
    assert not presentation.root.exists()
    # The replaced file should contain the data from the source file
    assert dst_file.read_text("utf8") == "# Test"
    # The moved files from source should be equal to the
    # files on destination directory
    assert src_files == dst_files


def test_extract_file_zipfile(
    presentation: Presentation,
    presentation_zip: Path,
) -> None:
    src_files = sorted(f.name for f in presentation.root.iterdir())

    extracted_dir = Path(extract_file(presentation_zip, presentation.parent))

    extracted_files = sorted(f.name for f in extracted_dir.iterdir())

    assert extracted_files == src_files


def test_extract_file_tarfile(
    presentation: Presentation,
    presentation_tar: Path,
) -> None:
    src_files = sorted(f.name for f in presentation.root.iterdir())

    extracted_dir = Path(extract_file(presentation_tar, presentation.parent))

    extracted_files = sorted(f.name for f in extracted_dir.iterdir())

    assert extracted_files == src_files


def test_extract_file_on_non_file(tmp_path: Path) -> None:
    with pytest.raises(FileNotFoundError):
        extract_file(tmp_path / "notfound", tmp_path)


def test_extract_file_on_non_tar_or_zip(tmp_path: Path) -> None:
    wrong_format = tmp_path / "file.wrong"
    wrong_format.write_text("", "utf8")

    with pytest.raises(NotImplementedError):
        extract_file(wrong_format, tmp_path)


def test_make_presentation(tmp_path: Path) -> None:
    presentation = Presentation(tmp_path)

    make_presentation(presentation.root)

    assert presentation.root.is_dir()
    assert presentation.file.is_file()
    assert presentation.media.is_dir()
    assert presentation.config.is_file()



"""
Revelation presentation settings

This is an automatic generated template for revelation presentations
with the default options.
"""

REVEAL_META = {
    # Title of the slide
    "title": "The title",
    # Author in the metadata of the slide
    "author": "Some Author",
    # Description in the metadata of the slide
    "description": "Some description",
}

# Reveal markdown slide separator
REVEAL_SLIDE_SEPARATOR = "---"
REVEAL_VERTICAL_SLIDE_SEPARATOR = "---~"

# Themes
# beige, black, blood, league, moon, night, serif, simple, sky,
# solarized, white, dracula
REVEAL_THEME = "black"

REVEAL_CONFIG = {
    # Display presentation control arrows
    "controls": True,
    # Help the user learn the controls by providing hints, for example by
    # bouncing the down arrow when they first encounter a vertical slide
    "controlsTutorial": True,
    # Determines where controls appear, "edges" or "bottom-right"
    "controlsLayout": "bottom-right",
    # Visibility rule for backwards navigation arrows; "faded", "hidden"
    # or "visible"
    "controlsBackArrows": "faded",
    # Display a presentation progress bar
    "progress": True,
    # Display the page number of the current slide
    # - True:    Show slide number
    # - False:   Hide slide number
    #
    # Can optionally be set as a string that specifies the number formatting:
    # - "h.v":   Horizontal . vertical slide number (default)
    # - "h/v":   Horizontal / vertical slide number
    # - "c":   Flattened slide number
    # - "c/t":   Flattened slide number / total slides
    #
    # Alternatively, you can provide a function that returns the slide
    # number for the current slide. The function should take in a slide
    # object and return an array with one string [slideNumber] or
    # three strings [n1,delimiter,n2]. See #formatSlideNumber().
    "slideNumber": False,
    # Can be used to limit the contexts in which the slide number appears
    # - "all":      Always show the slide number
    # - "print":    Only when printing to PDF
    # - "speaker":  Only in the speaker view
    "showSlideNumber": "all",
    # Use 1 based indexing for # links to match slide number (default is zero
    # based)
    "hashOneBasedIndex": False,
    # Add the current slide number to the URL hash so that reloading the
    # page/copying the URL will return you to the same slide
    "hash": True,
    # Flags if we should monitor the hash and change slides accordingly
    "respondToHashChanges": True,
    # Push each slide change to the browser history
    "history": True,
    # Enable keyboard shortcuts for navigation
    "keyboard": True,
    # Optional function that blocks keyboard events when retuning false
    #
    # If you set this to 'focused', we will only capture keyboard events
    # for embedded decks when they are in focus
    "keyboardCondition": None,
    # Disables the default reveal.js slide layout (scaling and centering)
    # so that you can use custom CSS layout
    "disableLayout": False,
    # Enable the slide overview mode
    "overview": True,
    # Vertical centering of slides
    "center": True,
    # Enables touch navigation on devices with touch input
    "touch": True,
    # Loop the presentation
    "loop": False,
    # Change the presentation direction to be RTL
    "rtl": False,
    # Changes the behavior of our navigation directions.
    #
    # "default"
    # Left/right arrow keys step between horizontal slides, up/down
    # arrow keys step between vertical slides. Space key steps through
    # all slides (both horizontal and vertical).
    #
    # "linear"
    # Removes the up/down arrows. Left/right arrows step through all
    # slides (both horizontal and vertical).
    #
    # "grid"
    # When this is enabled, stepping left/right from a vertical stack
    # to an adjacent vertical stack will land you at the same vertical
    # index.
    #
    # Consider a deck with six slides ordered in two vertical stacks:
    # 1.1    2.1
    # 1.2    2.2
    # 1.3    2.3
    #
    # If you're on slide 1.3 and navigate right, you will normally move
    # from 1.3 -> 2.1. If "grid" is used, the same navigation takes you
    # from 1.3 -> 2.3.
    "navigationMode": "default",
    # Randomizes the order of slides each time the presentation loads
    "shuffle": False,
    # Turns fragments on and off globally
    "fragments": True,
    # Flags whether to include the current fragment in the URL,
    # so that reloading brings you to the same fragment position
    "fragmentInURL": True,
    # Flags if the presentation is running in an embedded mode,
    # i.e. contained within a limited portion of the screen
    "embedded": False,
    # Flags if we should show a help overlay when the questionmark
    # key is pressed
    "help": True,
    # Flags if it should be possible to pause the presentation (blackout)
    "pause": True,
    # Flags if speaker notes should be visible to all viewers
    "showNotes": False,
    # Global override for autolaying embedded media (video/audio/iframe)
    # - None:   Media will only autoplay if data-autoplay is present
    # - True:   All media will autoplay, regardless of individual setting
    # - False:  No media will autoplay, regardless of individual setting
    "autoPlayMedia": None,
    # Global override for preloading lazy-loaded iframes
    # - None:   Iframes with data-src AND data-preload will be loaded
    #           when within the viewDistance, iframes with only data-src
    #           will be loaded when visible
    # - True:   All iframes with data-src will be loaded when within
    #           the viewDistance
    # - False:  All iframes with data-src will be loaded only when visible
    "preloadIframes": None,
    # Can be used to globally disable auto-animation
    "autoAnimate": True,
    # Optionally provide a custom element matcher that will be
    # used to dictate which elements we can animate between.
    "autoAnimateMatcher": None,
    # Default settings for our auto-animate transitions, can be
    # overridden per-slide or per-element via data arguments
    "autoAnimateEasing": "ease",
    "autoAnimateDuration": 1.0,
    "autoAnimateUnmatched": True,
    # CSS properties that can be auto-animated. Position & scale
    # is matched separately so there's no need to include styles
    # like top/right/bottom/left, width/height or margin.
    "autoAnimateStyles": [
        "opacity",
        "color",
        "background-color",
        "padding",
        "font-size",
        "line-height",
        "letter-spacing",
        "border-width",
        "border-color",
        "border-radius",
        "outline",
        "outline-offset",
    ],
    # Controls automatic progression to the next slide
    # - 0:      Auto-sliding only happens if the data-autoslide HTML attribute
    #           is present on the current slide or fragment
    # - 1+:     All slides will progress automatically at the given interval
    # - False:  No auto-sliding, even if data-autoslide is present
    "autoSlide": 0,
    # Stop auto-sliding after user input
    "autoSlideStoppable": True,
    # Use this method for navigation when auto-sliding
    # (defaults to navigateNext)
    "autoSlideMethod": None,
    # Specify the average time in seconds that you think you will spend
    # presenting each slide. This is used to show a pacing timer in the
    # speaker view
    "defaultTiming": None,
    # Enable slide navigation via mouse wheel
    "mouseWheel": False,
    # Opens links in an iframe preview overlay
    # Add `data-preview-link` and `data-preview-link="false"` to customise
    # each link individually
    "previewLinks": False,
    # Exposes the reveal.js API through window.postMessage
    "postMessage": True,
    # Dispatches all reveal.js events to the parent window through postMessage
    "postMessageEvents": False,
    # Focuses body when page changes visibility to ensure keyboard
    # shortcuts work
    "focusBodyOnPageVisibilityChange": True,
    # Transition style
    "transition": "slide",  # none/fade/slide/convex/concave/zoom
    # Transition speed
    "transitionSpeed": "default",  # default/fast/slow
    # Transition style for full page slide backgrounds
    "backgroundTransition": "fade",  # none/fade/slide/convex/concave/zoom
    # Prints each fragment on a separate slide
    "pdfSeparateFragments": True,
    # Offset used to reduce the height of content within exported PDF pages.
    # This exists to account for environment differences based on how you
    # print to PDF. CLI printing options, like phantomjs and wkpdf, can end
    # on precisely the total height of the document whereas in-browser
    # printing has to end one pixel before.
    "pdfPageHeightOffset": -1,
    # Number of slides away from the current that are visible
    "viewDistance": 3,
    # Number of slides away from the current that are visible on mobile
    # devices. It is advisable to set this to a lower number than
    # viewDistance in order to save resources.
    "mobileViewDistance": 2,
    # The display mode that will be used to show slides
    "display": "block",
    # Hide cursor if inactive
    "hideInactiveCursor": True,
    # Time before the cursor is hidden (in ms)
    "hideCursorTime": 5000,
}

REVEAL_PLUGINS = {
    # Enable markdown plugin. To disable set `"Markdown": {}`
    "Markdown": {
        # Name of plugin in `plugins` field for `Reveal.initialize`
        "initialize_as": "RevealMarkdown",
        "scripts": [
            "markdown/markdown.js",
        ],
    },
    # Enable and configure math plugin. To disable set `"Math": {}`
    "Math": {
        # Name of plugin in `plugins` field for `Reveal.initialize`
        "initialize_as": "RevealMath.KaTeX",
        "scripts": [
            "math/math.js",
        ],
    },
    # Enable highlight plugin. To disable set `"Highlight": {}`
    "Highlight": {
        "initialize_as": "RevealHighlight",
        "scripts": [
            "highlight/highlight.js",
        ],
        "styles": [
            "highlight/monokai.css",
        ],
    },
    # Enable search plugin. To disable set `"Search": {}`
    "Search": {
        "initialize_as": "RevealSearch",
        "scripts": [
            "search/search.js",
        ],
    },
    # Enable notes plugin. To disable set `"Notes": {}`
    "Notes": {
        "initialize_as": "RevealNotes",
        "scripts": [
            "notes/notes.js",
        ],
    },
    # Enable zoom plugin. To disable set `"Zoom": {}`
    "Zoom": {
        "initialize_as": "RevealZoom",
        "scripts": [
            "zoom/zoom.js",
        ],
    },
}



"""Revelation configuration handler"""

from pathlib import Path
from types import ModuleType

from revelation import default_config


class Config(dict):
    """
    Class that stores revelation configs thanks to flask Config:

    https://github.com/pallets/flask/blob/master/flask/config.py

    It loads the default_config variables and if a path is passed it also
    loads the external configs
    """

    def __init__(self, filename: Path | None = None):
        """Initializes the config with the defaults or with custom
        variables from an external file"""
        self.load_from_object(default_config)

        if filename and filename.is_file():
            self.load_from_pyfile(filename)

    def load_from_object(self, obj: ModuleType) -> None:
        """Load the configs from a python object passed
        to the function"""
        for key in dir(obj):
            if key.isupper():
                self[key] = getattr(obj, key)

    def load_from_pyfile(self, filename: Path) -> None:
        """Load the configs from a python file as it was imported"""
        module = ModuleType("config")
        module.__file__ = str(filename)

        try:
            with filename.open(mode="rb") as fp:
                exec(compile(fp.read(), filename, "exec"), module.__dict__)
        except OSError as error:
            strerror = error.strerror
            error.strerror = f"Unable to load configuration file ({strerror})"

            raise error

        self.load_from_object(module)



from pathlib import Path

from platformdirs import user_data_dir

REVEAL_URL = "https://github.com/hakimel/reveal.js/archive/master.zip"

STATIC_ROOT = Path(user_data_dir(appname="revelation")) / "static"

REVEALJS_DIR = STATIC_ROOT / "revealjs"



"""
Main revelation module

It has the Revelation main class that creates the webserver do run
the presentation
"""

from __future__ import annotations

import re
import typing
from collections.abc import Iterable
from pathlib import Path

from jinja2 import Environment, PackageLoader, select_autoescape
from werkzeug.middleware.shared_data import SharedDataMiddleware
from werkzeug.wrappers import Request, Response

from revelation.config import Config
from revelation.constants import STATIC_ROOT
from revelation.utils import normalize_newlines

if typing.TYPE_CHECKING:
    from _typeshed.wsgi import StartResponse, WSGIEnvironment


class Revelation:
    """
    Main revelation app class that instantiates the server and handles
    the requests
    """

    presentation: Path
    config: Config
    media: Path | None
    theme: Path | None
    style: Path | None

    def __init__(
        self,
        presentation: Path,
        config: Path | None = None,
        media: Path | None = None,
        theme: Path | None = None,
        style: Path | None = None,
    ) -> None:
        """
        Initializes the server and creates the environment for the presentation
        """
        self.presentation = presentation
        self.config = Config(config)
        self.media = media
        self.theme = theme
        self.style = style

        shared_data = {}
        shared_data.update(self.parse_shared_data(STATIC_ROOT))
        shared_data.update(self.parse_shared_data(self.media))
        shared_data.update(self.parse_shared_data(self.theme))
        shared_data.update(self.parse_shared_data(self.style))

        self.wsgi_app = SharedDataMiddleware(self._wsgi_app, shared_data)

    def parse_shared_data(self, shared_root: Path | None) -> dict:
        """
        Parse additional shared_data if it exists
        """
        if shared_root and shared_root.exists():
            shared_root = shared_root.resolve()
            shared_url = f"/{shared_root.name}"

            return {shared_url: str(shared_root)}

        return {}

    def load_slides(
        self,
        path: Path,
        section_separator: str,
        vertical_separator: str,
    ) -> list:
        """
        Get slides file from the given path, loads it and split into list
        of slides.

        :return: a list of strings with the slides content
        """
        with path.open("rb") as fp:
            slides = normalize_newlines(fp.read().decode("utf-8"))

        return [
            re.split(f"^{vertical_separator}$", section, flags=re.MULTILINE)
            for section in re.split(
                f"^{section_separator}$", slides, flags=re.MULTILINE
            )
        ]

    def get_theme(self, theme_name: str) -> str:
        fullpath_theme = (
            STATIC_ROOT.resolve() / "revealjs" / "dist" / "theme" / f"{theme_name}.css"
        )

        if fullpath_theme.is_file():
            return f"static/revealjs/dist/theme/{theme_name}.css"

        return theme_name

    def dispatch_request(self, _: Request | None = None) -> Response:
        env = Environment(
            loader=PackageLoader("revelation", "templates"),
            autoescape=select_autoescape(["html"]),
        )

        context = {
            "meta": self.config.get("REVEAL_META"),
            "slides": self.load_slides(
                self.presentation,
                str(self.config.get("REVEAL_SLIDE_SEPARATOR")),
                str(self.config.get("REVEAL_VERTICAL_SLIDE_SEPARATOR")),
            ),
            "config": self.config.get("REVEAL_CONFIG"),
            "theme": self.get_theme(str(self.config.get("REVEAL_THEME"))),
            "style": getattr(self.style, "name", None),
            "plugins": self.config.get("REVEAL_PLUGINS"),
        }

        template = env.get_template("presentation.html")

        return Response(
            template.render(**context), headers={"content-type": "text/html"}
        )

    def _wsgi_app(
        self, environ: WSGIEnvironment, start_response: StartResponse
    ) -> Iterable[bytes]:
        request = Request(environ)
        response = self.dispatch_request(request)

        return response(environ, start_response)

    def __call__(
        self, environ: WSGIEnvironment, start_response: StartResponse
    ) -> Iterable[bytes]:
        return self.wsgi_app(environ, start_response)



from pathlib import Path
from typing import Annotated

from typer import Option

RevealUrl = Annotated[str, Option("--url", "-u", help="Reveal.js download url")]
ServerPort = Annotated[int, Option("--port", "-p", help="Presentation server port")]
ConfigFile = Annotated[Path | None, Option("--config", "-c", help="Custom config file")]
MediaDir = Annotated[Path | None, Option("--media", "-m", help="Custom media folder")]
ThemeDir = Annotated[Path | None, Option("--theme", "-t", help="Custom theme folder")]
StyleOverrideFile = Annotated[
    Path | None,
    Option(
        "--style-override-file",
        "-s",
        help="Custom css file to override reveal.js styles",
    ),
]
OutputFile = Annotated[
    Path, Option("--output-file", "-f", help="File name of the static presentation")
]
OutputFolder = Annotated[
    Path,
    Option(
        "--output-folder",
        "-o",
        help="Folder where the static presentation will be generated",
    ),
]
OverwriteOutputFlag = Annotated[
    bool, Option("--force", "-r", help="Overwrite the output folder if exists")
]
DebugFlag = Annotated[
    bool, Option("--debug", "-d", help="Run the revelation server on debug mode")
]



from pathlib import Path

from pytest_mock import MockerFixture
from typer.testing import CliRunner

from revelation.cli import cli

from .conftest import Presentation


def test_mkpresentation(tmp_path: Path) -> None:
    presentation = Presentation(tmp_path)

    runner = CliRunner()
    result = runner.invoke(cli, ["mkpresentation", str(presentation.root)])

    assert result.exit_code == 0
    assert presentation.root.is_dir()
    assert presentation.file.is_file()
    assert presentation.media.is_dir()
    assert presentation.config.is_file()


def test_mkpresentation_already_exists(presentation: Presentation) -> None:
    runner = CliRunner()
    result = runner.invoke(cli, ["mkpresentation", str(presentation.root)])

    assert result.exit_code == 1
    assert f"'{presentation.root}' already exists" in result.output


def test_mkstatic(presentation: Presentation) -> None:
    output_dir = presentation.parent / "output"
    index_file = output_dir / "index.html"
    static_dir = output_dir / "static"
    media_dir = output_dir / "media"

    runner = CliRunner()
    result = runner.invoke(
        cli, ["mkstatic", str(presentation.file), "-o", str(output_dir)]
    )

    assert result.exit_code == 0
    assert output_dir.is_dir()
    assert index_file.is_file()
    assert static_dir.is_dir()
    assert media_dir.is_dir()


def test_mkstatic_custom_media(presentation: Presentation) -> None:
    presentation_media = presentation.root / "custom_media"
    presentation_media.mkdir()
    presentation_media_file = presentation_media / "custom_file.txt"
    presentation_media_file.write_text("", "utf8")

    output_dir = presentation.parent / "output"
    custom_media_file = output_dir / "media" / presentation_media_file.name

    runner = CliRunner()
    result = runner.invoke(
        cli,
        [
            "mkstatic",
            str(presentation.file),
            "-o",
            str(output_dir),
            "-m",
            str(presentation_media),
        ],
    )

    assert result.exit_code == 0
    assert output_dir.is_dir()
    assert custom_media_file.is_file()


def test_mkstatic_without_media(presentation: Presentation) -> None:
    output_dir = presentation.parent / "output"

    presentation.media.rmdir()

    runner = CliRunner()
    result = runner.invoke(
        cli, ["mkstatic", str(presentation.file), "-o", str(output_dir)]
    )

    assert result.exit_code == 0
    assert "Media folder not detected, running without media" in result.output


def test_mkstatic_without_config(presentation: Presentation) -> None:
    output_dir = presentation.parent / "output"

    presentation.config.unlink()

    runner = CliRunner()
    result = runner.invoke(
        cli, ["mkstatic", str(presentation.file), "-o", str(output_dir)]
    )

    assert result.exit_code == 0
    assert "Configuration file not detected, running with defaults" in result.output


def test_mkstatic_with_theme(presentation: Presentation) -> None:
    (presentation.root / "theme").mkdir()

    output_dir = presentation.parent / "output"
    theme_dir = output_dir / "theme"

    runner = CliRunner()
    result = runner.invoke(
        cli, ["mkstatic", str(presentation.file), "-o", str(output_dir)]
    )

    assert result.exit_code == 0
    assert theme_dir.is_dir()


def test_mkstatic_override_styles(presentation: Presentation) -> None:
    style_file = presentation.root / "style.css"
    style_file.write_text("h1 { color: #000 }", "utf8")

    output_dir = presentation.parent / "output"
    output_style_file = output_dir / "style.css"

    runner = CliRunner()
    result = runner.invoke(
        cli,
        [
            "mkstatic",
            str(presentation.file),
            "-o",
            str(output_dir),
            "-s",
            str(style_file),
        ],
    )

    assert result.exit_code == 0
    assert output_style_file.is_file()


def test_mkstatic_output_already_exists_file(presentation: Presentation) -> None:
    output_file = presentation.parent / "output"
    output_file.write_text("", "utf8")

    runner = CliRunner()
    result = runner.invoke(
        cli, ["mkstatic", str(presentation.file), "-o", str(output_file)]
    )

    assert result.exit_code == 1
    assert f"'{output_file}' already exists and is a file" in result.output


def test_mkstatic_output_already_exists_folder(presentation: Presentation) -> None:
    output_dir = presentation.parent / "output"
    output_dir.mkdir()

    runner = CliRunner()
    result = runner.invoke(
        cli, ["mkstatic", str(presentation.file), "-o", str(output_dir)]
    )

    assert result.exit_code == 1
    assert f"'{output_dir}' already exists, use --force to override it" in result.output


def test_mkstatic_override_output(presentation: Presentation) -> None:
    output_dir = presentation.parent / "output"
    output_dir.mkdir()

    override_file = output_dir / "override.txt"
    override_file.write_text("", "utf8")

    runner = CliRunner()
    result = runner.invoke(
        cli, ["mkstatic", str(presentation.file), "-r", "-o", str(output_dir)]
    )

    assert result.exit_code == 0
    assert not override_file.exists()


def test_mkstatic_presentation_not_found(tmp_path: Path) -> None:
    presentation = tmp_path / "notfound"

    runner = CliRunner()
    result = runner.invoke(cli, ["mkstatic", str(presentation)])

    assert result.exit_code == 1
    assert "Presentation file not found" in result.output


def test_mkstatic_style_not_file(presentation: Presentation) -> None:
    style_dir = presentation.root / "style.css"
    style_dir.mkdir()

    runner = CliRunner()
    result = runner.invoke(
        cli, ["mkstatic", str(presentation.file), "-s", str(style_dir)]
    )

    assert result.exit_code == 1
    assert "Style is not a css file or does not exists" in result.output


def test_mkstatic_style_not_css(presentation: Presentation) -> None:
    style_file = presentation.root / "style.notcss"
    style_file.write_text("", "utf8")

    runner = CliRunner()
    result = runner.invoke(
        cli, ["mkstatic", str(presentation.file), "-s", str(style_file)]
    )

    assert result.exit_code == 1
    assert "Style is not a css file or does not exists" in result.output


def test_start(mocker: MockerFixture, presentation: Presentation) -> None:
    mocked_run_simple = mocker.patch("revelation.cli.run_simple")

    runner = CliRunner()
    runner.invoke(cli, ["start", str(presentation.file)])

    assert mocked_run_simple.called


def test_start_presentation_not_found(tmp_path: Path) -> None:
    presentation = tmp_path / "notfound"

    runner = CliRunner()
    result = runner.invoke(cli, ["start", str(presentation)])

    assert result.exit_code == 1
    assert "Presentation file not found" in result.output


def test_start_style_not_file(presentation: Presentation) -> None:
    style_dir = presentation.root / "style.css"
    style_dir.mkdir()

    runner = CliRunner()
    result = runner.invoke(cli, ["start", str(presentation.file), "-s", str(style_dir)])

    assert result.exit_code == 1
    assert "Style is not a css file or does not exists" in result.output


def test_start_style_not_css(presentation: Presentation) -> None:
    style_file = presentation.root / "style.notcss"
    style_file.write_text("", "utf8")

    runner = CliRunner()
    result = runner.invoke(
        cli, ["start", str(presentation.file), "-s", str(style_file)]
    )

    assert result.exit_code == 1
    assert "Style is not a css file or does not exists" in result.output



"""Utility tools used by revelation"""

import os
import shutil
import tarfile
import zipfile
from http.client import HTTPMessage
from pathlib import Path
from urllib.request import urlretrieve

from revelation import default_config


def make_presentation(presentation_path: Path) -> None:
    """
    Make a new presentation boilerplate code given a presentation_path
    """
    name = presentation_path.name

    (presentation_path / "media").mkdir(parents=True)

    shutil.copy(default_config.__file__, presentation_path / "config.py")

    with (presentation_path / "slides.md").open("w") as fp:
        title = name.replace("_", " ").replace("-", " ").title()

        fp.write(f"# {title}\n\nStart from here!")


def download_file(url: str) -> tuple[Path, HTTPMessage]:
    """
    Download a file from a given url
    """
    downloaded_file, result = urlretrieve(url)

    return Path(downloaded_file), result


def move_and_replace(src: Path, dst: Path) -> None:
    """
    Helper function used to move files from one place to another,
    creating os replacing them if needed

    :param src: source directory
    :param dst: destination directory
    """

    src = src.resolve()
    dst = dst.resolve()

    for src_dir, _, files in os.walk(src):
        # using os walk to navigate through the directory tree
        # keep the dir structure by replacing the source root to
        # the destination on walked path
        dst_dir = src_dir.replace(str(src), str(dst))

        if not os.path.exists(dst_dir):
            # to prevent copy from failing, create the not existing dirs
            os.makedirs(dst_dir)

        for file_ in files:
            src_file = os.path.join(src_dir, file_)
            dst_file = os.path.join(dst_dir, file_)

            if os.path.exists(dst_file):
                os.remove(dst_file)  # to copy not fail, create existing files

            shutil.move(src_file, dst_dir)  # move the files

    shutil.rmtree(src)  # remove the dir structure from the source


def extract_file(compressed_file: Path, path: Path = Path(".")) -> Path:
    """Extract function to extract from zip or tar file"""
    path = path.resolve()

    if compressed_file.is_file():
        if tarfile.is_tarfile(str(compressed_file)):
            with tarfile.open(compressed_file, "r:gz") as tfile:
                basename = tfile.getnames()[0]
                kwargs: dict = {
                    "numeric_owner": False,
                }

                if hasattr(tarfile, "data_filter"):
                    kwargs["filter"] = "data"

                tfile.extractall(str(path.resolve()), None, **kwargs)
        elif zipfile.is_zipfile(compressed_file):
            with zipfile.ZipFile(compressed_file, "r") as zfile:
                basename = zfile.namelist()[0]
                zfile.extractall(path)
        else:
            msg = "File type not supported"
            raise NotImplementedError(msg)
    else:
        msg = f"{compressed_file} is not a valid file"
        raise FileNotFoundError(msg)

    return path / basename


def normalize_newlines(text: str) -> str:
    """Normalize text to follow Unix newline pattern"""
    return text.replace("\r\n", "\n").replace("\r", "\n")



from werkzeug.test import Client
from werkzeug.wrappers import Response

from revelation import Revelation

from .conftest import Presentation


def test_parse_shared_data_empty(revelation: Revelation) -> None:
    shared_data_config = revelation.parse_shared_data(None)

    assert shared_data_config == {}


def test_parse_shared_data(presentation: Presentation, revelation: Revelation) -> None:
    shared_data_config = revelation.parse_shared_data(presentation.media)

    assert shared_data_config == {
        f"/{presentation.media.name}": str(presentation.media)
    }


def test_load_slides(presentation: Presentation, revelation: Revelation) -> None:
    presentation.file.write_text("# Pag1\n---\n# Pag2.1\n---~\n# Page2.2", "utf8")

    slides = revelation.load_slides(presentation.file, "---", "---~")

    assert slides == [["# Pag1\n"], ["\n# Pag2.1\n", "\n# Page2.2"]]


def test_load_slides_non_normalized(
    presentation: Presentation, revelation: Revelation
) -> None:
    presentation.file.write_bytes(b"# Pag1\r---\r\n# Pag2")

    slides = revelation.load_slides(presentation.file, "---", "---~")

    assert slides == [["# Pag1\n"], ["\n# Pag2"]]


def test_load_slides_non_ascii(
    presentation: Presentation, revelation: Revelation
) -> None:
    presentation.file.write_text("# こんにちは\n---\n# 乾杯", "utf8")

    slides = revelation.load_slides(presentation.file, "---", "---~")

    assert slides == [["# こんにちは\n"], ["\n# 乾杯"]]


def test_client_request_ok(revelation: Revelation) -> None:
    client = Client(revelation, Response)

    response = client.get("/")

    assert response.status == "200 OK"
    assert response.headers.get("Content-Type") == "text/html"
